@@ -6,6 +6,7 @@
  * Some ICCCM manager selection code derived from fvwm2,
  * Copyright (C) 2001 Dominik Vogt, Matthias Clasen, and fvwm2 team
  * Copyright (C) 2003 Rob Adams
+ * Copyright (C) 2004 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -581,14 +582,16 @@ meta_screen_new (MetaDisplay *display,
   reload_xinerama_infos (screen);
   
   meta_screen_set_cursor (screen, META_CURSOR_DEFAULT);
+  
+  if (display->no_focus_window == None)
+    {
+      display->no_focus_window = meta_create_offscreen_window (display->xdisplay,
+                                                               screen->xroot);
 
-  /* Handle creating a no_focus_window for this screen */  
-  screen->no_focus_window = meta_create_offscreen_window (display->xdisplay,
-                                                          screen->xroot);
-  XSelectInput (display->xdisplay, screen->no_focus_window,
-                FocusChangeMask | KeyPressMask | KeyReleaseMask);
-  XMapWindow (display->xdisplay, screen->no_focus_window);
-  /* Done with no_focus_window stuff */
+      XSelectInput (display->xdisplay, display->no_focus_window,
+                    FocusChangeMask | KeyPressMask | KeyReleaseMask);
+      XMapWindow (display->xdisplay, display->no_focus_window);
+    }
   
   set_wm_icon_size_hint (screen);
   
