@@ -4,6 +4,7 @@
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2002 Red Hat, Inc.
  * Copyright (C) 2003, 2004 Rob Adams
+ * Copyright (C) 2004, 2005 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -124,6 +125,7 @@ struct _MetaWindow
 
   /* Minimize is the state controlled by the minimize button */
   guint minimized : 1;
+  guint tab_unminimized : 1;
 
   /* Whether the window is mapped; actual server-side state
    * see also unmaps_pending
@@ -207,6 +209,9 @@ struct _MetaWindow
   /* Have we placed this window? */
   guint placed : 1;
 
+  /* Is this not a transient of the focus window which is being denied focus? */
+  guint denied_focus_and_not_transient : 1;
+
   /* Has this window not ever been shown yet? */
   guint showing_for_first_time : 1;
 
@@ -429,6 +434,14 @@ void        meta_window_get_outer_rect       (MetaWindow    *window,
 void        meta_window_get_xor_rect         (MetaWindow          *window,
                                               const MetaRectangle *grab_wireframe_rect,
                                               MetaRectangle       *xor_rect);
+void        meta_window_begin_wireframe (MetaWindow *window);
+void        meta_window_update_wireframe (MetaWindow *window,
+                                          int         x,
+                                          int         y,
+                                          int         width,
+                                          int         height);
+void        meta_window_end_wireframe (MetaWindow *window);
+
 void        meta_window_delete             (MetaWindow  *window,
                                             Time         timestamp);
 void        meta_window_kill               (MetaWindow  *window);
@@ -529,4 +542,7 @@ void meta_window_queue_update_icon (MetaWindow *window);
 void meta_window_stack_just_below (MetaWindow *window,
                                    MetaWindow *below_this_one);
 
+void meta_window_set_user_time (MetaWindow *window,
+                                Time        timestamp);
+
 #endif
